`include "common.v"
`include "LIF.v"

module SNN_top
  #(
    parameter LIF_NEURON_OVRFL_WIDTH = 8,
    parameter SPK_ARRAY_WIDTH = 784,
    parameter NEURAL_ARRAY_WIDTH = 48,
    parameter OUTPUT_ARRAY_WIDTH = 10,
    parameter WEIGHT_ADDR_WIDTH = 10,
    parameter WEIGHT_WIDTH = 8,
    parameter AXI4_ADDR_WIDTH = 16,
    parameter AXI4_DATA_WIDTH = 32
    )(
      input			    s_axi_aclk,
      input			    s_axi_aresetn,
      input			    s_axi_awvalid,
      input [AXI4_ADDR_WIDTH-1:0]   s_axi_awaddr,
      input [2:0]		    s_axi_awprot,
      output			    s_axi_awready,

      input			    s_axi_wvalid,
      input [AXI4_DATA_WIDTH-1:0]   s_axi_wdata,
      input [AXI4_DATA_WIDTH/8-1:0] s_axi_wstrb,
      output			    s_axi_wready,
      
      output			    s_axi_bvalid,
      output [1:0]		    s_axi_bresp,
      input			    s_axi_bready,

      input			    s_axi_arvalid,
      input [AXI4_ADDR_WIDTH-1:0]   s_axi_araddr,
      input [2:0]		    s_axi_arprot,
      output			    s_axi_arready,

      output			    s_axi_rvalid,
      output [AXI4_DATA_WIDTH-1:0]  s_axi_rdata,
      output [1:0]		    s_axi_rresp,
      input			    s_axi_rready
      );

   wire				    w_start;
   wire [9:0] 			    w_spike;
   wire				    w_svalid;
   wire [3:0] 			    w_number;
   wire 			    w_done;

   SNN_Slave_If_v1_1_S00_AXI
     #(
       .C_S_AXI_DATA_WIDTH(AXI4_DATA_WIDTH),
       .C_S_AXI_ADDR_WIDTH(AXI4_ADDR_WIDTH)
       )
   slave_if
     (
      .i_spike(w_spike),
      .i_number(w_number),
      .i_valid(w_svalid),
      .i_done(w_done),
      .o_start(w_start),
      .S_AXI_ACLK(s_axi_aclk),
      .S_AXI_ARESETN(s_axi_aresetn),
      .S_AXI_AWADDR(s_axi_awaddr),
      .S_AXI_AWPROT(s_axi_awprot),
      .S_AXI_AWVALID(s_axi_awvalid),
      .S_AXI_AWREADY(s_axi_awready),
      .S_AXI_WDATA(s_axi_wdata),
      .S_AXI_WSTRB(s_axi_wstrb),
      .S_AXI_WVALID(s_axi_wvalid),
      .S_AXI_WREADY(s_axi_wready),
      .S_AXI_BRESP(s_axi_bresp),
      .S_AXI_BVALID(s_axi_bvalid),
      .S_AXI_BREADY(s_axi_bready),
      .S_AXI_ARADDR(s_axi_araddr),
      .S_AXI_ARVALID(s_axi_arvalid),
      .S_AXI_ARREADY(s_axi_arready),
      .S_AXI_ARPROT(s_axi_arprot),
      .S_AXI_RDATA(s_axi_rdata),
      .S_AXI_RRESP(s_axi_rresp),
      .S_AXI_RVALID(s_axi_rvalid),
      .S_AXI_RREADY(s_axi_rready)
      );

   SNN_wrapper
     #(
       .LIF_NEURON_OVRFL_WIDTH(LIF_NEURON_OVRFL_WIDTH),
       .SPK_ARRAY_WIDTH(SPK_ARRAY_WIDTH),
       .NEURAL_ARRAY_WIDTH(NEURAL_ARRAY_WIDTH),
       .OUTPUT_ARRAY_WIDTH(OUTPUT_ARRAY_WIDTH),
       .WEIGHT_ADDR_WIDTH(WEIGHT_ADDR_WIDTH),
       .WEIGHT_WIDTH(WEIGHT_WIDTH)
       )
   SNN0
     (
      .clk(s_axi_aclk),
      .rst_n(s_axi_aresetn),
      .i_start(w_start),
      .o_spike(w_spike),
      .o_svalid(w_svalid),
      .o_number(w_number),
      .o_done(w_done)
      );
   
endmodule

	 
